/* $Workfile: OsaLinuxC.h $   $Revision: 4 $    $Modtime: 12/05/03 3:03p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc internal

  @module

  @normal Copyright (C) Odin TeleSystems Inc., 1998-2003, All Rights Reserved

*******************************************************************************/

/*---------------------------------------------------------------------------*\
  Note: starting from RedHat (kernel-2.4.18-14), OtxDrv gets compilation 
  errors, to cope with kernel and compiler (3.x) changes of RedHat 8.0 and 9.0, 
  functions using Linux kernel API in c++ files are segregated and put in 
  c files  
\*---------------------------------------------------------------------------*/

#ifndef OSA_LINUX_C_H
#define OSA_LINUX_C_H

#ifdef __cplusplus
extern "C" {
#endif

/*---------------------------------------------------------------------------*\
  
 from OsaLinuxCppRtl.h OsaLinuxCppRtl.cpp

\*---------------------------------------------------------------------------*/

#include "../OsaDef.h"
#include "OsaLinuxRtl.h"
                      
void OsaSpinLockImpl_Ctor(void **pSLock);
void OsaSpinLockImpl_Dtor(void **pSLock);
void OsaSpinLockImpl_Lock(void *pSLock, __u32 *flags);
void OsaSpinLockImpl_Unlock(void *pSLock, __u32 savedFlags);

void OsaEventImpl_Ctor(void **pWQHead);
void OsaEventImpl_Dtor(void **pWQHead);
void OsaEventImpl_wake_up(void **pWQHead);


/*---------------------------------------------------------------------------*\  
 from OsaLinux.h and OsaLinux.cpp
\*---------------------------------------------------------------------------*/
   
void OsaLinuxUserEventDevice_Ctor(void **pWQHead);
void OsaLinuxUserEventDevice_Dtor(void **pWQHead);
void OsaLinuxUserEventDevice_wake_up(void **pWQHead);
void OsaLinuxUserEventDevice_poll_wait(struct file *pFl, 
                                       void **pWQHead, 
                                       struct poll_table_struct *pPTable);


/*---------------------------------------------------------------------------*\  
 from OsaLinux.h and OsaLinux.cpp
\*---------------------------------------------------------------------------*/


#define new ident_new
#define delete ident_delete
#define class ident_class
#define virtual ident_virtual			//KERNEL_VERSION(2,4,0): conflict with "virtual" in header file <linux/mm.h>  
#define namespace ident_namespace

#include <asm/poll.h>       //POLLIN | POLLRDNORM | POLLOUT | POLLWRNORM;
#include <linux/ioport.h>   //for struct resource

#undef new
#undef delete
#undef class
#undef virtual		// KERNEL_VERSION(2,4,0)
#undef namespace	// KERNEL_VERSION(2,4,0)

// from <linux/pci.h>
#define   PCI_BASE_ADDRESS_0	0x10	/* 32 bits */ 
#define   PCI_BASE_ADDRESS_SPACE_IO 0x01 
#define   PCI_BASE_ADDRESS_MEM_MASK	(~0x0fUL) 
#define   PCI_BASE_ADDRESS_IO_MASK	(~0x03UL) 
#define   PCI_SUBSYSTEM_ID	0x2e 
#define   DEVICE_COUNT_COMPATIBLE	4
#define   DEVICE_COUNT_IRQ	2
#define   DEVICE_COUNT_DMA	2
#define   DEVICE_COUNT_RESOURCE	12

  
typedef struct list_head_test {
  struct list_head_test *next, *prev;
}list_test;

// match <linux/pci.h>
struct pci_dev_otx{

  struct list_head_test global_list;	
	struct list_head_test bus_list;	
	struct pci_bus	*bus;		
	struct pci_bus	*subordinate;

	void		*sysdata;	
	struct proc_dir_entry *procent;	

	unsigned int	devfn;		
	unsigned short	vendor;
	unsigned short	device;
	unsigned short	subsystem_vendor;
	unsigned short	subsystem_device;
	unsigned int	class_otx;		
	u8		hdr_type;	
	u8		rom_base_reg;	

	struct pci_driver *driver;	
	void		*driver_data;	
	u64		dma_mask;	
	u32             current_state;  
	
	unsigned short vendor_compatible[DEVICE_COUNT_COMPATIBLE];
	unsigned short device_compatible[DEVICE_COUNT_COMPATIBLE];

	unsigned int	irq;
	struct resource resource[DEVICE_COUNT_RESOURCE]; 
	struct resource dma_resource[DEVICE_COUNT_DMA];
	struct resource irq_resource[DEVICE_COUNT_IRQ];

	char		name[80];
	char		slot_name[8];	
	int		active;	
	int		ro;		
	unsigned short	regs;		

	int (*prepare)(struct pci_dev_otx *dev);	
	int (*activate)(struct pci_dev_otx *dev);
	int (*deactivate)(struct pci_dev_otx *dev);
};
  


int   OsaLinux_pci_present();  
void  OsaLinux_pci_read_config_word(struct pci_dev_otx *dev, int where, u16 *val);
void  OsaLinux_pci_read_config_dword(struct pci_dev_otx *dev, int where, u32 *val);
void  OsaLinux_pci_write_config_dword(struct pci_dev_otx *dev, int where, u32 val);

struct pci_dev_otx * OsaLinux_pci_dev_g_fun1(void);
struct pci_dev_otx * OsaLinux_pci_dev_g_fun2(void);
struct pci_dev_otx * OsaLinux_pci_dev_g_fun3(struct pci_dev_otx *pPciDevice);


/*---------------------------------------------------------------------------*\
  from OsaLinux.h and OsaLinux.cpp
  
  for RedHat 9.0; in RedHat 8.0, it compiles without modifying followings
\*---------------------------------------------------------------------------*/

void OsaLinux_MOD_INC_USE_COUNT(void);
void OsaLinux_MOD_DEC_USE_COUNT(void);

const void * Osalinux_inter_module_get(const char *sModule);
void OsaLinux_inter_module_register(const char *sName, const void *pFunName);
void OsaLinux_inter_module_unregister(const char *sName);

#ifdef __cplusplus
}
#endif

#endif //OSA_LINUX_C_H
